#!/bin/bash
# **********************************************************************
# $File: serverstats.sh $
# $Author: mschulz $ - $Date: 2023-03-09 12:26:09 +0100 (Mi, 09 März 2023) $
# $Description: Script to collect memory/cpu statistics of the current server maschine. $
# **********************************************************************

# Define the idle time for this service. You can define 1d => 24h or 86400 seconds.
interval="5m"

# The path to the output logfile, where the statistics metrics will be storend
outfile="/var/log/serverstats.log"

# The default logformat. Each %name is a stat_<name> function.
# You can add parameters to some functions, like %<function>_<arg1>_<arg2>.
logformat="%timestamp %hostname %loadavg %memoryusage %memorycached %swapusage %membuffer %cpuusage %diskspace %traffic"

# Unit for the interface traffic. You can define mb or kb. default is bytes.
inet_unit="kb"

# Load configuration, when exists
if [ -f /etc/default/serverstats ]; then
	. /etc/default/serverstats

	[ -n "${INTERVAL}" ] && interval=${INTERVAL}
	[ -n "${OUTPUTFILE}" ] && outfile=${OUTPUTFILE}
	[ -n "${LOGFORMAT}" ] && logformat=${LOGFORMAT}
	[ -n "${UNIT}" ] && inet_unit=${UNIT}
fi

# Get the default interface with ip
inet_iface=$(ip -4 route ls | grep default | grep -Po '(?<=dev )(\S+)' | head -n1)

# **********************************************************************
# This function converts time designations such as
# 1d or 12h into seconds. Supported are Xd, Xh, Xm, Xs.
#
# @param $1 - The time designations
# @return init
#
str2seconds () {
 
local item=$1
local digit=$(echo $item | perl -n -e 'print $1 if ($_ =~ m/^(\d+)((?:d|h|m|s))$/g);')
local unit=$(echo $item | perl -n -e 'print $2 if ($_ =~ m/^(\d+)((?:d|h|m|s))$/g);')
local sec
 
case $unit in
	d) sec=$((${digit}*86400)) ;;
	h) sec=$((${digit}*3600))  ;;
	m) sec=$((${digit}*60))    ;;
	s) sec=${digit}            ;;
esac
 
echo $sec
 
}

# **********************************************************************
# This function print out the timestamp for the record.
#
# @return string
#
stat_timestamp () {

# Print out the date in format like [09/Mar/2023:00:56:04 +0100]
echo -n "[$(date +%d/%m/%Y\ %H:%M:%S\ %z)] "

}

# **********************************************************************
# This function print out the hostname for the record.
#
# @return string
#
stat_hostname () {

# Print out the hostname
echo -n "\"$(hostname)\" "

}

# **********************************************************************
# This function get the network traffic statistics for an defined interface.
#
# @param $1 - The interface name
# @param $2 - The unit of the traffic (mb,kb,bytes)
# @param $3 - Verbose on|off
# @return string
#
stat_traffic () {

local iface=$1
local unit=${2:-"b"}
local verbose=${3:-"on"}
local label="Bytes/s"

local prefix="traffic,"

# Get received and transmited bytes of the current interface
local rxold=$(cat /sys/class/net/${iface}/statistics/rx_bytes 2>/dev/null)
local txold=$(cat /sys/class/net/${iface}/statistics/tx_bytes 2>/dev/null)

# We wait one second
sleep 1

# Get received and transmited bytes of the current interface
local rxnew=$(cat /sys/class/net/${iface}/statistics/rx_bytes 2>/dev/null)
local txnew=$(cat /sys/class/net/${iface}/statistics/tx_bytes 2>/dev/null)

# Calculate received and transmited Byte/KB/MB/ per second of the current interface
if [ $unit = "KB" -o $unit = "kb" ]; then
        tx=$(echo "$txnew $txold" | perl -a -n -e 'printf("%.2f", (($F[0]-$F[1])/1024));')
        rx=$(echo "$rxnew $rxold" | perl -a -n -e 'printf("%.2f", (($F[0]-$F[1])/1024));')
        label="KB/s"
elif [ $unit = "MB" -o $unit = "mb" ]; then
        tx=$(echo "$txnew $txold" | perl -a -n -e 'printf("%.2f", (($F[0]-$F[1])/131072));')
        rx=$(echo "$rxnew $rxold" | perl -a -n -e 'printf("%.2f", (($F[0]-$F[1])/131072));')
        label="MB/s"
else
        tx=$(echo "$txnew $txold" | perl -a -n -e 'printf("%.2f", ($F[0]-$F[1]));')
        rx=$(echo "$rxnew $rxold" | perl -a -n -e 'printf("%.2f", ($F[0]-$F[1]));')
fi

if [ $verbose = "on" ]; then
        local suffix="RX $iface: $rx ${label}, TX $iface: $tx ${label}"
else
        local suffix="net_rx=${rx},net_tx=${tx}"
fi

echo -n "\"${prefix}${suffix}\" "

}

# **********************************************************************
# This function get the Load Average (1,5,15) statistics.
#
# @return string
#
stat_loadavg () {

local prefix="loadavg,"
local suffix=$(awk '{print "loadavg1="$1",loadavg5="$2",loadavg15="$3}' /proc/loadavg)
echo -n "\"${prefix}${suffix}\" "

}

# **********************************************************************
# This function get the Physical Memory Usage (total, free) statistics.
#
# @return string
#
stat_memoryusage () {

local memory=$(awk '$0 ~ /^(MemFree|MemTotal)/{ printf "%%s=%s,", $2 }' /proc/meminfo | sed 's/,$//g')
local prefix="memory,"
local suffix=$(printf "${memory}" "mem_total" "mem_free")
echo -n "\"${prefix}${suffix}\" "

}

# **********************************************************************
# This function get the Cached Memory Usage (free, buffers, cache) statistics.
#
# @return string
#
stat_memorycached () {

local memcache=$(awk '$0 ~ /^(MemFree|Buffers|Cached)/{ printf "%%s=%s,", $2 }' /proc/meminfo | sed 's/,$//g')
local prefix="memcache,"
local suffix=$(printf "${memcache}" "mem_free" "mem_buffers" "mem_cache")
echo -n "\"${prefix}${suffix}\" "

}

# **********************************************************************
# This function get the Swap Usage (total, free) statistics.
#
# @return string
#
stat_swapusage () {

local swapusage=$(awk '$0 ~ /^(SwapFree|SwapTotal)/{ printf "%%s=%s,", $2 }' /proc/meminfo | sed 's/,$//g')
local prefix="swapusage,"
local suffix=$(printf "${swapusage}" "swap_total" "swap_free")
echo -n "\"${prefix}${suffix}\" "

}

# **********************************************************************
# This function get the Memory Buffers (total, buffers) statistics.
#
# @return string
#
stat_membuffer () {

local membuffer=$(awk '$0 ~ /^(Buffers|MemTotal)/{ printf "%%s=%s,", $2 }' /proc/meminfo | sed 's/,$//g')
local prefix="membuffer,"
local suffix=$(printf "${membuffer}" "mem_total" "mem_buffers")
echo -n "\"${prefix}${suffix}\" "

}

# **********************************************************************
# This function get the Used Space / (total, used) statistics.
#
# @param $1 - The mountpoint, default is /
# @return string
#
stat_diskspace () {

local mount=${1:-"/"}
local label=${mount}

# Mask shlashes
mount=$( echo ${mount} | sed 's!\/!\\\/!g' )

local suffix=$(df | awk '$6 ~ /^'${mount}'$/ {print "space_total="$2",space_used="$3}')
local prefix="usedspace_${label},"
echo -n "\"${prefix}${suffix}\" "

}

# **********************************************************************
# This function get the CPU Usage (user,nice,system,idle,iowait,irq) statistics.
#
# @return string
#
stat_cpuusage () {

local prefix="cpuusage,"
local cpu=$(top -n1 -b | grep '%Cpu(s)' | sed 's/\,/\./g')
local system=$(echo $cpu | awk '{printf "%.2f", $4}')
local user=$(echo $cpu | awk '{printf "%.2f", $2}')
local nice=$(echo $cpu | awk '{printf "%.2f", $6}')
local total=$(awk '{u=$2+$4; t=$2+$4+$5; if (NR==1){u1=u; t1=t;} else printf "%.2f\n", ($2+$4-u1) * 100 / (t-t1); }' <(grep 'cpu ' /proc/stat) <(sleep 1;grep 'cpu ' /proc/stat))
local suffix="system=${system},user=${user},nice=${nice},total=${total}"
echo -n "\"${prefix}${suffix}\" "

}

# **********************************************************************
# MAIN

# Get the interval as seconds
interval=$(str2seconds "${interval}")

# Check, if any interface was defined for traffic statistics (1=yes|0=no)
default=$( echo ${logformat} | perl -ne 'print ($_ =~ m/\%traffic_\w+/g && "1" || "0")' )

# Set default parameter for network traffic
if [ ${default} -eq 0 ]; then
	logformat=$( echo ${logformat} | perl -p -n -e 's#\%traffic#\%traffic_'${inet_iface}'_'${inet_unit}'#g')
fi

# Start endless loop
while true; do

	# Collect metrics and write it to output file
	(
	for item in ${logformat}; do
		eval "$(echo ${item} | perl -F_ -a -n -e '$func = shift @F; $func =~ s/\%//g; print "stat_",$func, " @F";')"
	done
	echo
	) >>${outfile}

	# Sleep for X seconds
	sleep ${interval}

done

# vim: syntax=sh ts=4 sw=4 sts=4 sr noet
# EOF
